<?php
/**
 * @file
 * Form implementation.
 */

/**
 * Interface for an extension to build the profile entity form.
 */
interface ActonProfileFormDelegateInterface {
  /**
   * Builds the edit form for a profile.
   *
   * @param array $form
   *   Existing form array to build on.
   * @param array $form_state
   *   Form state by reference.
   * @param ActonProfile $profile
   *   Site profile for which to build edit form.
   * @param string $operation
   *   Form operation name.
   * @return array
   *   Profile edit form array, including trigger elements (e.g. buttons).
   */
  public function form($form, &$form_state, ActonProfile $profile, $operation);

  /**
   * Validates an edit form submission.
   */
  public function validate($form, &$form_state);

  /**
   * Processes an edit form submission.
   */
  public function submit($form, &$form_state);
}

/**
 * Profile entity form builder extension for the default style controller.
 */
class ActonProfileDefaultFormDelegate implements ActonProfileFormDelegateInterface {
  /**
   * {@inheritdoc}
   */
  public function form($form, &$form_state, ActonProfile $profile, $operation) {
    // Show profile prefill form.
    if ($operation == 'add-site-id' && !isset($form_state['acton_profile'])) {
      $form = $this->prefillForm($form, $form_state);
      return $form;
    }

    // Prepare form parameters.
    if (!isset($form_state['acton_profile'])) {
      $form_state['acton_profile'] = $profile;
      if ($operation == 'clone') {
        // Reset for cloning.
        $profile->name .= ' (cloned)';
        $profile->machine_name = '';
      }
    }
    $form_state['operation'] = $operation;

    // Build form.
    $form = $this->profileForm($form, $form_state);
    return $form;
  }

  /**
   * Builds the prefill form.
   */
  protected function prefillForm($form, &$form_state) {
    $form['site_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Site ID'),
      '#default_value' => isset($form_state['values']['site_id']) ? $form_state['values']['site_id'] : '',
      '#description' => t('Search for profile with this site ID. A !listing can be found on the Web Publishing website.', array(
        '!listing' => l(t('listing of site IDs'), 'http://webpublishing.anu.edu.au/steps/registering/site-ids.php', array('attributes' => array('target' => '_blank'))),
      )),
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Add profile'),
      '#name' => 'prefill',
    );

    return $form;
  }

  /**
   * Builds the profile form.
   */
  protected function profileForm($form, &$form_state) {
    $form = $this->basicForm($form, $form_state);
    $form = $this->dataForm($form, $form_state);
    $form = $this->settingsForm($form, $form_state);
    return $form;
  }

  /**
   * Adds the basic profile form elements, i.e. for name, buttons, etc.
   */
  protected function basicForm($form, &$form_state) {
    $profile = $form_state['acton_profile'];
    $operation = $form_state['operation'];

    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Profile name'),
      '#default_value' => $profile->name,
      '#required' => TRUE,
      '#weight' => -20,
    );
    $form['machine_name'] = array(
      '#type' => 'machine_name',
      '#default_value' => $profile->machine_name,
      '#maxlength' => 64,
      '#machine_name' => array(
        'exists' => 'acton_profile_load',
      ),
      '#disabled' => $operation == 'edit',
      '#weight' => -20,
    );

    $form['actions'] = array(
      '#type' => 'actions',
      '#weight' => 40,
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save profile'),
    );

    return $form;
  }

  /**
   * Adds profile data properties form.
   */
  protected function dataForm($form, &$form_state) {
    // Set up data vertical tab set.
    $form['data'] = array(
      '#type' => 'vertical_tabs',
      '#title' => t('Profile data'),
      '#tree' => TRUE,
      '#default_tab' => 'basic',
    );

    // Build form for default properties.
    $form = $this->defaultDataForm($form, $form_state);

    return $form;
  }

  /**
   * Adds data form for default profile properties.
   */
  protected function defaultDataForm($form, &$form_state) {
    form_load_include($form_state, 'inc', 'acton_profile', 'includes/acton_profile.form');
    $form['#attached']['css'][] = drupal_get_path('module', 'acton_profile') . '/acton_profile.css';

    $profile = $form_state['acton_profile'];
    $operation = $form_state['operation'];

    // Prepare elements.
    $elements = array();
    module_load_include('inc', 'acton_profile', 'acton_profile.info');
    $data = acton_profile_default_data_properties();
    foreach ($data as $key => $property) {
      // Skip computed properties.
      if (!empty($property['computed'])) {
        continue;
      }

      // Prepare value.
      $value = isset($profile->data[$key]) ? $profile->data[$key] : '';
      if ($property['type'] == 'date') {
        // Transform value from timestamp to readable format.
        if ($value) {
          $value = date('j M Y', $value);
        }
        // Insert today.
        else {
          $value = date('j M Y');
        }
      }

      // Create element.
      $elements[$key] = array(
        '#type' => 'textfield',
        '#title' => filter_xss($property['label']),
        '#default_value' => $value,
        '#description' => filter_xss($property['description']),
        '#required' => !empty($property['required']),
        '#parents' => array('data', $key),
      );
      if ($property['type'] == 'date') {
        $elements[$key]['#element_validate'][] = 'acton_profile_form_element_date_validate';
      }
    }

    // Adapt site URL element based on default values from current request state.
    global $is_https;
    $elements['site_url']['#field_prefix'] = $is_https ? 'https://' : 'http://';
    if ($operation == 'add' && empty($profile->data['site_url'])) {
      $elements['site_url']['#default_value'] = $_SERVER['HTTP_HOST'];
    }

    // Set up field sets.
    $fieldsets = array(
      'info' => t('Basic information'),
      'contacts' => t('Contacts'),
      'metadata' => t('Metadata'),
      'misc' => t('Miscellaneous'),
    );
    $fields = array(
      'info' => array(
        'site_short_name',
        'site_id',
        'site_url',
      ),
      'contacts' => array(
        'resp_officer',
        'resp_officer_address',
        'site_contact',
        'site_contact_address',
      ),
      'metadata' => array(
        'date_modified',
        'meta_description',
        'meta_subject',
      ),
      'misc' => array(
        'about_page',
      ),
    );

    // Construct the form elements.
    foreach ($fieldsets as $fieldset_key => $fieldset_title) {
      $form['data'][$fieldset_key] = array(
        '#type' => 'fieldset',
        '#title' => check_plain($fieldset_title),
      );
      foreach ($fields[$fieldset_key] as $field_key) {
        $form['data'][$fieldset_key][$field_key] = $elements[$field_key];
      }
    }

    return $form;
  }

  /**
   * Adds profile settings form.
   */
  protected function settingsForm($form, &$form_state) {
    $profile = $form_state['acton_profile'];

    $profile->settings += array(
      'style_version' => '',
      'search_show' => FALSE,
      'search_base' => array(),
      'search_advanced' => TRUE,
      'server_override' => FALSE,
      'server_endpoint_base' => '',
      'server_endpoint_suffix' => '',
    );

    // Add settings fieldset.
    $form['settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Settings'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
    );

    // Add basic settings.
    $form['settings']['metadata_use'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use profile for page metadata.'),
      '#description' => t('Acton Profile can provide the following metadata: identifier, title, description, subject, date modified, creator, creator email.'),
      '#default_value' => !empty($profile->settings['metadata_use']),
      '#parents' => array('settings', 'metadata_use'),
    );
    $form['settings']['style_version'] = array(
      '#type' => 'textfield',
      '#title' => t('Overridden style version'),
      '#size' => 3,
      '#default_value' => $profile->settings['style_version'],
      '#element_validate' => array('element_validate_integer_positive'),
      '#description' => t('Specify a style version to override the default provided by the style server. Leave blank to use the default version for this site profile.'),
    );

    // Add search settings.
    $form['settings']['search'] = array(
      '#type' => 'fieldset',
      '#title' => t('Search box'),
    );
    $form['settings']['search']['search_show'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show search box'),
      '#default_value' => !empty($profile->settings['search_show']),
      '#description' => t('Display the ANU search box to enable searching with ANU search engine by Funnelback. This is different from the Drupal search box.'),
      '#parents' => array('settings', 'search_show'),
    );
    $form['settings']['search']['search_base'] = array(
      '#type' => 'textarea',
      '#title' => t('Search base URLs'),
      '#default_value' => !empty($profile->settings['search_base']) ? implode("\n", $profile->settings['search_base']) : '',
      '#description' => t('The URLs under which to return Funnelback search results, one per line. Do not include the protocol (e.g. http://). Only works if ANU search is enabled, above. If nothing is entered, this defaults to the Site URL (above). If you have a different URL, or more than one, for search results you MUST set this value to the desired domain(s).'),
      '#states' => array(
        'visible' => array(
          ':input[name="settings[search_show]"]' => array('checked' => TRUE),
        ),
      ),
      '#parents' => array('settings', 'search_base'),
      '#element_validate' => array('acton_profile_form_element_split_lines'),
    );
    $form['settings']['search']['search_advanced'] = array(
      '#type' => 'checkbox',
      '#title' => t('Advanced search link'),
      '#default_value' => !empty($profile->settings['search_advanced']),
      '#description' => t('Check this option to display a link to the advanced search form.'),
      '#states' => array(
        'visible' => array(
          ':input[name="settings[search_show]"]' => array('checked' => TRUE),
        ),
      ),
      '#parents' => array('settings', 'search_advanced'),
    );

    // Add style server settings.
    $form['settings']['server'] = array(
      '#type' => 'fieldset',
      '#title' => t('Style server'),
    );
    $form['settings']['server']['server_override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override default style server'),
      '#default_value' => !empty($profile->settings['server_override']),
      '#parents' => array('settings', 'server_override'),
    );
    $server_override_states = array(
      'visible' => array(
        ':input[name="settings[server_override]"]' => array('checked' => TRUE),
      ),
    );
    $form['settings']['server']['server_endpoint_base'] = array(
      '#type' => 'textfield',
      '#title' => t('Style server base'),
      '#default_value' => $profile->settings['server_endpoint_base'],
      '#description' => t('Use this style server base instead of the default base. The default base is: !base', array('!base' => '<strong>' . variable_get('acton_profile_default_endpoint_base', ACTON_PROFILE_DEFAULT_ENDPOINT_BASE) . '</strong>')),
      '#field_prefix' => $GLOBALS['is_https'] ? 'https://' : 'http://',
      '#states' => $server_override_states,
      '#parents' => array('settings', 'server_endpoint_base'),
    );
    $form['settings']['server']['server_endpoint_suffix'] = array(
      '#type' => 'textfield',
      '#title' => t('Style server suffix'),
      '#default_value' => $profile->settings['server_endpoint_suffix'],
      '#description' => t('Use this style server suffix instead of the default suffix. The default suffix is: !suffix', array('!suffix' => '<strong>' . variable_get('acton_profile_default_endpoint_suffix', ACTON_PROFILE_DEFAULT_ENDPOINT_SUFFIX) . '</strong>')),
      '#states' => $server_override_states,
      '#parents' => array('settings', 'server_endpoint_suffix'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validate($form, &$form_state) {
    if ($form_state['triggering_element']['#name'] == 'prefill') {
      $this->validatePrefill($form, $form_state);
    }
    else {
      $this->validateProfile($form, $form_state);
    }
  }

  /**
   * Validates prefill form submission.
   */
  protected function validatePrefill($form, &$form_state) {
    // Build profile.
    $defaults['data']['site_id'] = $form_state['values']['site_id'];
    $profile = entity_create('acton_profile', $defaults);

    // Obtain site info.
    $controller = acton_profile_style_controller();
    if ($controller instanceof ActonProfileStyleInfoInterface && $info = $controller->getSiteInfo($profile)) {
      // Validate site ID.
      if (!$info->isValid()) {
        form_set_error('site_id', t('The ID cannot be found on the style server.'));
      }
      // Create profile with prefilled data.
      else {
        // Build profile.
        $sitevars = isset($info['sitevars']) ? $info['sitevars'] : array();
        $sitevars += array(
          'sitename' => '',
          'site_id' => '',
          'site_url' => '',
        );
        $defaults['name'] = decode_entities($sitevars['sitename']);
        $defaults['data']['site_short_name'] = $defaults['name'];
        $defaults['data']['site_url'] = trim($sitevars['site_url'], '/');
        $profile = entity_create('acton_profile', $defaults);

        $form_state['acton_profile'] = $profile;
        $form_state['rebuild'] = TRUE;
      }
    }
    else {
      form_set_error('site_id', t('Style information is not available with the currently configured style controller.'));
    }
  }

  /**
   * Validates profile form submission.
   */
  protected function validateProfile($form, &$form_state) {
    $data = $form_state['values']['data'];
    $settings = $form_state['values']['settings'];
    $profile = entity_ui_form_submit_build_entity($form, $form_state);
    $controller = acton_profile_style_controller();

    // Validate the server endpoint.
    $validEndpoint = TRUE;
    if ($controller instanceof ActonProfileStyleEndpointInterface) {
      $endpoint = $controller->getEndpoint($profile);
      if (!$endpoint->isValid()) {
        form_set_error('settings][server_endpoint_base', t('The style server is invalid.'));
        form_set_error('settings][server_endpoint_suffix');
        $validEndpoint = FALSE;
      }
    }

    // Validate backend info.
    if ($validEndpoint && variable_get('acton_profile_validate_site_id', TRUE) && $controller instanceof ActonProfileStyleInfoInterface) {
      $info = $controller->getSiteInfo($profile);
      if (!$info || !$info->isValid()) {
        form_set_error('data][site_id', t('The site ID is invalid.'));
      }
    }

    // Validate the site URL.
    if (!$this->isBaseUrlValid($data['site_url'])) {
      form_set_error('data][site_url', t('The site URL is invalid.'));
    }

    // Validate profile data.
    $this->validateProfileData($form, $form_state, $profile);

    // Validate search base URLs.
    foreach ($settings['search_base'] as $searchBase) {
      if (!$this->isBaseUrlValid($searchBase)) {
        form_set_error('settings][search_base', t('One or more of the specified search base URLs are invalid.'));
        break;
      }
    }
  }

  /**
   * Validates profile data.
   */
  protected function validateProfileData($form, &$form_state, ActonProfile $profile) {
    $data = $profile->data + array(
      'resp_officer_address' => '',
      'site_contact_address' => '',
    );

    // Validate contact addresses.
    if ($data['resp_officer_address'] && !valid_acton_profile_contact_address($data['resp_officer_address'])) {
      form_set_error('data][resp_officer_address', t('The responsible officer URL/email should contain a valid URL or email address.'));
    }
    if ($data['site_contact_address'] && !valid_acton_profile_contact_address($data['site_contact_address'])) {
      form_set_error('data][site_contact_address', t('The site contact URL/email should contain a valid URL or email address.'));
    }
  }

  /**
   * Validates base URL.
   */
  protected function isBaseUrlValid($baseUrl) {
    if (!$baseUrl || preg_match('`^[^:]+://`', $baseUrl)) {
      return FALSE;
    }
    return valid_url('http://' . $baseUrl);
  }

  /**
   * {@inheritdoc}
   */
  public function submit($form, &$form_state) {
    $this->cleanSubmit($form, $form_state);

    // Save profile.
    $profile = entity_ui_form_submit_build_entity($form, $form_state);
    $profile->save();

    drupal_set_message(t('The profile has been saved.'));
    $form_state['redirect'] = 'admin/appearance/acton-profile';

    acton_profile_clear_cache();
  }

  /**
   * Cleans submitted values.
   */
  protected function cleanSubmit($form, &$form_state) {
    // Clean data.
    if (isset($form_state['values']['data']['data__active_tab'])) {
      unset($form_state['values']['data']['data__active_tab']);
    }

    // Trim data values.
    foreach ($form_state['values']['data'] as $key => $value) {
      if (is_string($value)) {
        $form_state['values']['data'][$key] = trim($value);
      }
    }
  }
}

/**
 * Interface for settings form controller.
 */
interface ActonProfileSettingsFormControllerInterface {
  /**
   * Builds the settings form.
   *
   * @param array $form
   *   Existing form array to build on.
   * @param array $form_state
   *   Form state by reference.
   * @return array
   *   Settings form array.
   */
  public function form($form, &$form_state);

  /**
   * Validates an edit form submission.
   */
  public function validate($form, &$form_state);

  /**
   * Processes an edit form submission.
   */
  public function submit($form, &$form_state);
}

/**
 * Default controller for form configuring general style settings.
 */
class ActonProfileDefaultSettingsFormController implements ActonProfileSettingsFormControllerInterface {
  /**
   * {@inheritdoc}
   */
  public function form($form, &$form_state) {
    global $is_https;

    $form['server'] = array(
      '#type' => 'fieldset',
      '#title' => t('Style server settings'),
    );
    $form['server']['acton_profile_default_endpoint_base'] = array(
      '#type' => 'textfield',
      '#title' => t('Default style server URL base'),
      '#description' => t('This style server URL base will be used by default. In addition, it will be used to perform JSON pre-fill requests. This option can be overridden in a profile.'),
      '#field_prefix' => $is_https ? 'https://' : 'http://',
      '#default_value' => variable_get('acton_profile_default_endpoint_base', ACTON_PROFILE_DEFAULT_ENDPOINT_BASE),
    );
    $form['server']['acton_profile_default_endpoint_suffix'] = array(
      '#type' => 'textfield',
      '#title' => t('Default style server URL suffix'),
      '#description' => t('This suffix is used by default after the URL base above to fetch style page parts. In addition, it will be used to perform JSON pre-fill requests. This option can be overridden in a profile.'),
      '#default_value' => variable_get('acton_profile_default_endpoint_suffix', ACTON_PROFILE_DEFAULT_ENDPOINT_SUFFIX),
    );
    $form['server']['acton_profile_validate_site_id'] = array(
      '#type' => 'checkbox',
      '#title' => t('Validate site ID when saving profile'),
      '#description' => t('When saving a profile, this option specifies whether or not a provided site ID should be checked against the style server.'),
      '#default_value' => variable_get('acton_profile_validate_site_id', TRUE),
    );

    $form['cache'] = array(
      '#type' => 'fieldset',
      '#title' => t('Remote style cache settings'),
    );
    $form['cache']['acton_profile_cache'] = array(
      '#type' => 'checkbox',
      '#title' => t('Cache remote styles separately'),
      '#description' => t('Whether to cache fetched remote styles to ignore site-wide cache flushes.'),
      '#default_value' => variable_get('acton_profile_cache', FALSE),
    );
    $cache_element_states = array(
      'visible' => array(
        ':checkbox[name="acton_profile_cache"]' => array('checked' => TRUE),
      ),
    );
    $form['cache']['acton_profile_cache_lifetime'] = array(
      '#type' => 'textfield',
      '#title' => t('Cache lifetime'),
      '#default_value' => variable_get('acton_profile_cache_lifetime', ACTON_PROFILE_DEFAULT_CACHE_LIFE),
      '#description' => t('How long (in seconds) before this cache expires.'),
      '#element_validate' => array('element_validate_integer_positive'),
      '#states' => $cache_element_states,
    );
    $form['cache']['flush_now'] = array(
      '#type' => 'submit',
      '#value' => t('Flush cache now'),
      '#name' => 'flush_now',
      '#states' => $cache_element_states,
    );

    return system_settings_form($form);
  }

  /**
   * {@inheritdoc}
   */
  public function validate($form, &$form_state) {
    $base = $form_state['values']['acton_profile_default_endpoint_base'];
    $suffix = $form_state['values']['acton_profile_default_endpoint_suffix'];
    $endpoint = ActonProfileEndpoint::fromBaseSuffix($base, $suffix);
    if (!$endpoint->isValid()) {
      form_set_error('acton_profile_default_endpoint_base', t('The specified style server is invalid.'));
      form_set_error('acton_profile_default_endpoint_suffix');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submit($form, &$form_state) {
    acton_profile_clear_cache();
    if ($form_state['triggering_element']['#name'] == 'flush_now') {
      drupal_set_message(t('Remote style cache has successfully been flushed.'));
    }
  }
}

/**
 * Validates date element.
 */
function acton_profile_form_element_date_validate($element, &$form_state) {
  if ($element['#value']) {
    $date = date_parse($element['#value']);
    if (empty($date) || !empty($date['errors'])) {
      form_error($element, t('The specified date in %element is invalid.', array('%element' => $element['#title'])));
    }
    else {
      $value = mktime(0, 0, 0, $date['month'], $date['day'], $date['year']);
      form_set_value($element, $value, $form_state);
    }
  }
}

/**
 * Splits textarea input into an array of lines.
 */
function acton_profile_form_element_split_lines($element, &$form_state) {
  $value = $element['#value'];
  $value = array_filter(preg_split('/[\r\n]+/', trim($value)));
  form_set_value($element, $value, $form_state);
}
